@@ -148,6 +148,31 @@ shell_notify_online_cb (EShell *shell)
 	e_passwords_set_online (online);
 }
 
+static void
+shell_window_removed_cb (EShell *shell)
+{
+	g_return_if_fail (E_IS_SHELL (shell));
+
+	if (!gtk_application_get_windows (GTK_APPLICATION (shell)) &&
+	    !shell->priv->ready_to_quit)
+		e_shell_quit (shell, E_SHELL_QUIT_LAST_WINDOW);
+}
+
+static gboolean
+shell_window_delete_event_cb (GtkWindow *window,
+                              GdkEvent *event,
+                              GtkApplication *application)
+{
+	/* If other windows are open we can safely close this one. */
+	if (g_list_length (gtk_application_get_windows (application)) > 1)
+		return FALSE;
+
+	/* Otherwise we initiate application quit. */
+	e_shell_quit (E_SHELL (application), E_SHELL_QUIT_LAST_WINDOW);
+
+	return TRUE;
+}
+
 static void
 shell_action_new_window_cb (GSimpleAction *action,
                             GVariant *parameter,
@@ -427,15 +452,6 @@ shell_ready_for_quit (EShell *shell,
 	/* Finalize the activity. */
 	g_object_unref (activity);
 
-	/* XXX Inhibiting session manager actions currently only
-	 *     works on GNOME, so check that we obtained a valid
-	 *     inhibit cookie before attempting to uninhibit. */
-	if (shell->priv->inhibit_cookie > 0) {
-		gtk_application_uninhibit (
-			application, shell->priv->inhibit_cookie);
-		shell->priv->inhibit_cookie = 0;
-	}
-
 	if (shell->priv->prepare_quit_timeout_id) {
 		g_source_remove (shell->priv->prepare_quit_timeout_id);
 		shell->priv->prepare_quit_timeout_id = 0;
@@ -1433,10 +1449,7 @@ e_setup_theme_icons_theme_changed_cb (GtkSettings *gtk_settings)
 	GSettings *g_settings;
 	gboolean use_symbolic_icons = FALSE;
 	gboolean using_symbolic_icons;
-	gchar **paths = NULL;
 	gchar *icon_theme_name = NULL;
-	guint n_symbolic = 0, n_non_symbolic = 0;
-	guint ii;
 
 	g_settings = e_util_ref_settings ("org.gnome.evolution.shell");
 	prefer_symbolic_icons = g_settings_get_enum (g_settings, "prefer-symbolic-icons");
@@ -1460,43 +1473,38 @@ e_setup_theme_icons_theme_changed_cb (GtkSettings *gtk_settings)
 			"gtk-icon-theme-name", &icon_theme_name,
 			NULL);
 
-		gtk_icon_theme_get_search_path (icon_theme, &paths, NULL);
+		if (g_strcmp0 (icon_theme_name, "HighContrast") == 0 ||
+		    g_strcmp0 (icon_theme_name, "ContrastHigh") == 0) {
+			use_symbolic_icons = TRUE;
+		} else {
+			/* pick few common action icons and check whether they are
+			   only symbolic in the current theme */
+			const gchar *sample_icon_names[][3] = {
+				{ "appointment-new", "appointment-new-symbolic", NULL },
+				{ "edit-cut", "edit-cut-symbolic", NULL },
+				{ "edit-copy", "edit-copy-symbolic", NULL },
+				{ "mail-reply-sender", "mail-reply-sender-symbolic", NULL }
+			};
+			guint n_symbolic = 0, n_non_symbolic = 0;
+			guint ii;
 
-		for (ii = 0; paths && paths[ii]; ii++) {
-			GDir *dir;
-			gchar *dirname;
+			for (ii = 0; ii < G_N_ELEMENTS (sample_icon_names); ii++) {
+				GtkIconInfo *icon_info;
 
-			dirname = g_build_filename (paths[ii], icon_theme_name, "16x16", "actions", NULL);
-			dir = g_dir_open (dirname, 0, NULL);
-			if (dir) {
-				const gchar *filename;
+				icon_info = gtk_icon_theme_choose_icon (icon_theme, sample_icon_names[ii], 32, 0);
+				if (icon_info) {
+					if (gtk_icon_info_is_symbolic (icon_info))
+						n_symbolic++;
+					else
+						n_non_symbolic++;
 
-				for (filename = g_dir_read_name (dir);
-				     filename;
-				     filename = g_dir_read_name (dir)) {
-					/* pick few common action icons and check whether they are
-					   only symbolic in the current theme */
-					if (g_str_has_prefix (filename, "appointment-new") ||
-					    g_str_has_prefix (filename, "edit-cut") ||
-					    g_str_has_prefix (filename, "edit-copy")) {
-						if (strstr (filename, "-symbolic.") ||
-						    strstr (filename, ".symbolic.")) {
-							n_symbolic++;
-						} else {
-							n_non_symbolic++;
-						}
-					}
+					g_clear_object (&icon_info);
 				}
-				g_dir_close (dir);
 			}
-			g_free (dirname);
+
+			use_symbolic_icons = n_symbolic > n_non_symbolic;
 		}
 
-		use_symbolic_icons = (!n_non_symbolic && n_symbolic > 0) ||
-			g_strcmp0 (icon_theme_name, "HighContrast") == 0 ||
-			g_strcmp0 (icon_theme_name, "ContrastHigh") == 0;
-
-		g_strfreev (paths);
 		g_free (icon_theme_name);
 		break;
 	}
@@ -1791,6 +1799,8 @@ shell_finalize (GObject *object)
 {
 	EShell *self = E_SHELL (object);
 
+	g_warn_if_fail (self->priv->inhibit_cookie == 0);
+
 	g_hash_table_destroy (self->priv->backends_by_name);
 	g_hash_table_destroy (self->priv->backends_by_scheme);
 	g_hash_table_destroy (self->priv->auth_prompt_parents);
@@ -1826,6 +1836,10 @@ shell_constructed (GObject *object)
 
 	/* Chain up to parent's constructed() method. */
 	G_OBJECT_CLASS (e_shell_parent_class)->constructed (object);
+
+	g_signal_connect (
+		object, "window-removed",
+		G_CALLBACK (shell_window_removed_cb), NULL);
 }
 
 static void
@@ -1898,6 +1912,20 @@ shell_activate (GApplication *application)
 	e_shell_create_shell_window (E_SHELL (application), NULL);
 }
 
+static void
+shell_shutdown (GApplication *application)
+{
+	EShell *shell = E_SHELL (application);
+
+	if (shell->priv->inhibit_cookie > 0) {
+		gtk_application_uninhibit (GTK_APPLICATION (application), shell->priv->inhibit_cookie);
+		shell->priv->inhibit_cookie = 0;
+	}
+
+	/* Chain up to parent's method. */
+	G_APPLICATION_CLASS (e_shell_parent_class)->shutdown (application);
+}
+
 static void
 shell_window_added (GtkApplication *application,
                     GtkWindow *window)
@@ -1908,6 +1936,10 @@ shell_window_added (GtkApplication *application,
 	GTK_APPLICATION_CLASS (e_shell_parent_class)->
 		window_added (application, window);
 
+	g_signal_connect (
+		window, "delete-event",
+		G_CALLBACK (shell_window_delete_event_cb), application);
+
 	/* We use the window's own type name and memory
 	 * address to form a unique window role for X11. */
 	role = g_strdup_printf (
@@ -2268,6 +2300,7 @@ e_shell_class_init (EShellClass *class)
 	application_class = G_APPLICATION_CLASS (class);
 	application_class->startup = shell_startup;
 	application_class->activate = shell_activate;
+	application_class->shutdown = shell_shutdown;
 
 	gtk_application_class = GTK_APPLICATION_CLASS (class);
 	gtk_application_class->window_added = shell_window_added;
